#!/bin/ksh
#############################################################################
#############################################################################
## Licensed Materials - Property of IBM
## "Restricted Materials of IBM"
## 5724-O98
## (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with
## IBM Corp.
#############################################################################
## utility to deploy the database using the metadata files
## usage: deployMetadata.sh [ -migrate ]
##############################################################################

# include the RFIDIC environment settings
SCRIPT_DIR=`dirname $0`
if [ -n "$RFIDIC_HOME" -a -d $RFIDIC_HOME"/bin" ];then
	. ${RFIDIC_HOME}/bin/tsEnv.sh
	. ${RFIDIC_HOME}/bin/utils.sh
	. ${RFIDIC_HOME}/bin/prereqChecks.sh
else
	. ${SCRIPT_DIR}/tsEnv.sh
	. ${SCRIPT_DIR}/utils.sh
  . ${SCRIPT_DIR}/prereqChecks.sh
fi

logFileName=TSDeployLog_`date "+%Y%m%d%H%M%S"`
assignLogFile ${scriptDir}/../logs/$logFileName
absLogPathName="`cd \"$logDir\" 2>/dev/null && pwd || echo \"$logDir\"`/$logFileName"

# print usage
usage()
{
	 tsGetMessage.sh RFID05085I "$0"        
}

getWAScredentials()
{
    if [ -n "$was_admin_user" -a -z "$was_admin_password" ];then
        tsGetMessage.sh RFID05030I "$was_admin_user"
        stty -echo
        read was_admin_password
        stty echo
        echo
    fi

    if [ -n "$was_admin_user" -a -n "$was_admin_password" ];then
        credentials="-user $was_admin_user -password $was_admin_password"
    else
        credentials=""
    fi
}

# Starting applications
# Repeat message here so it gets written to the log.
startApplications()
{
    log " " 
    log "-------- Starting TS applications --------"
    getWAScredentials
    
    ## assign WAS security credentials
    if [ -n "$was_admin_user" -a -n "$was_admin_password" ];then
      parms="-was_admin_user=$was_admin_user -was_admin_password=$was_admin_password"
    else
      parms=""
    fi
   
    ${TS_HOME}/bin/startTS.sh -components=ENGINE,UI,SERIALID,CAPTURE $parms  >> $utilsLogFile 2>&1
    startTS_retCode=$?
    log "Return code from start applications is " $startTS_retCode
}

# Stopping applications
# Repeat message here so it gets written to the log.
stopApplications()
{
    log " " 
    log "-------- Stopping TS applications --------"
    getWAScredentials
    
    ## assign WAS security credentials
    if [ -n "$was_admin_user" -a -n "$was_admin_password" ];then
      parms="-was_admin_user=$was_admin_user -was_admin_password=$was_admin_password"
    else
      parms=""
    fi
    
    ${TS_HOME}/bin/stopTS.sh -components=ENGINE,UI,SERIALID,CAPTURE $parms  >> $utilsLogFile 2>&1
    stopTS_retCode=$?
    log "Return code from stop applications is " $stopTS_retCode
}

checkWAS()
{
    getWAScredentials
    if [ -z "$was_admin_user" -a -z "$was_admin_password" ];then
        isWASRunning
    else
        isWASRunning $was_admin_user $was_admin_password
    fi 
    was_status=$?
}


#### Main starts here #########
for item in $*
do
	if [ "$item" = "-migrate" ];then
		migrate="-migrate";
	elif [ "$item" = "-force_drop_tables" ];then
		drop_tables=true;
	elif [ `expr match "$item" '-dba_user'` != 0 ]; then
		dba_user=`$ECHO $item | $AWK_EQUAL '{print($2)}'`
	elif [ `expr match "$item" '-dba_password'` != 0 ]; then
		dba_password=`$ECHO $item | $AWK_EQUAL '{print($2)}'`
	elif [ "$item" = "-force_restart_apps" ];then
		restart_apps=true;
	elif [ `expr match "$item" '-was_admin_user'` != 0 ];then
		was_admin_user=`$ECHO $item | $AWK_EQUAL '{print($2)}'`
	elif [ `expr match "$item" '-was_admin_password'` != 0 ];then
		was_admin_password=`$ECHO $item | $AWK_EQUAL '{print($2)}'`
	elif [ `expr match "$item" '-metadata_dir'` != 0 ];then
		METADATA_FILES_PATH=`$ECHO $item | $AWK_EQUAL '{print($2)}'`
	else
		usage
		exit 1
	fi
done

checkWAS
if [ "$was_status" -eq "$TRUE" ];then
    if [ "$restart_apps" != "true" ];then
        tsGetMessage.sh -noid RFID05247I
        promptUserYesNo
        if [ "$?" -eq "$YES" ];then
            restart_apps=true
        else
            tsGetMessage.sh -noid RFID05255E
            exit 1
        fi
    fi

    if [ "$restart_apps" = "true" ];then
        echo
        echo "-------- Stopping TS applications --------"
        stopApplications
        
    	if [ "$stopTS_retCode" != "0" ];then
        	tsGetMessage.sh RFID05248E
            echo
            tsGetMessage.sh RFID97081E $absLogPathName
            echo
        	exit 1
    	fi
    fi
fi

checkLogs

if [ -z "$migrate" ];then
	if [ "$drop_tables" != "true" ];then
		tsGetMessage.sh -noid RFID05254E
		promptUserYesNo
		if [ "$?" -eq "$YES" ];then
			drop_tables=true
		else
			tsGetMessage.sh -noid RFID05255E
			exit 1
		fi
	fi
fi

isHostnameRemote $DBMS_HOST
db_dist=$?

run_command java -classpath $CLASSPATH -DRFIDIC_HOME=$RFIDIC_HOME -DTS_HOME=$RFIDIC_HOME com.ibm.rfidic.utils.install.DBConnect connect
conn=$?
if [ "$conn" = "0" ];then
	#echo "Connection to DB successful"
	tsGetMessage.sh RFID05238I
	if [ "$drop_tables" = "true" -a -z "$migrate" ];then
		# Just drop tables and recreate them if the database exists
		migrate="-drop_tables"
	fi
else
	#echo "Connection to DB was not successful. Exiting deploy..."
	tsGetMessage.sh RFID05239I
	exit 1
fi

if [ -n "$dba_user" -a -z "$dba_password" ];then
	tsGetMessage.sh -noid RFID05245E "$dba_user"
	stty -echo
	read dba_password
	stty echo
fi
if [ -n "$dba_user" -a -n "$dba_password" ];then
	dba_user_option="-dba_user="$dba_user
	dba_password_option="-dba_password="$dba_password
fi
if [ -z "$METADATA_FILES_PATH" ];then
	METADATA_FILES_PATH="$RFIDIC_HOME/etc"
fi

run_command java -classpath $CLASSPATH -DRFIDIC_HOME=$RFIDIC_HOME -DTS_HOME=$RFIDIC_HOME com.ibm.rfidic.metadata.deploy.Deploy $migrate $dba_user_option $dba_password_option >> $utilsLogFile 2>&1
retCode=$?

if [ "$retCode" = "0" ];then
    echo
    tsGetMessage.sh RFID90023I
    createObjectCaches >> $utilsLogFile 2>&1
    	if [ "$retCode" != "0" ];then
        	tsGetMessage.sh RFID05267E
        	exit 1
		fi 	        
else
    echo
    tsGetMessage.sh RFID90022E
    echo
    tsGetMessage.sh RFID97081E $absLogPathName
    echo
	exit 1
fi

if [ "$was_status" -eq "$TRUE" ];then
    echo
    echo "-------- Starting TS applications --------"
    startApplications
  
    if [ "$startTS_retCode" != "0" ];then
        tsGetMessage.sh RFID05268E
        echo
        tsGetMessage.sh RFID97081E $absLogPathName
        echo
        exit 1
    fi 	
fi

exit 0
