#!/bin/ksh
#############################################################################
## Licensed Materials - Property of IBM
## "Restricted Materials of IBM"
## 5724-O98
## (c)  Copyright IBM Corp. 2011 All Rights Reserved.
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with
## IBM Corp.
#############################################################################
##
##   Title:       importAddTSAR.sh 
##   Description: Utility to import Traceability Server artifacts, which are 
##                added to the existing ones.
##   Usage:       importAddTSAR.sh  -file <TSAR_file> 
##                                  [-was_admin_user <was_admin_user> 
##                                   -was_admin_password <was_admin_password>] 
##                                  [-was_runas_user <was_runas_user>] 
##                                  [-dba_user <dba_user> \n\
##                                   -dba_password <dba_password>] \n\
##
#############################################################################

# Get arguments used for working with WAS
INDEX=0
for arg in $@
do
  ARGUMENTS[$INDEX]=$arg
  let "INDEX += 1"
done

while [ $INDEX -gt 0 ]
do
  let "NEXT_INDEX = $INDEX"
  let "INDEX -= 1"
  if [ ${ARGUMENTS[$INDEX]} == "-was_admin_user" ];then
    was_admin_user=${ARGUMENTS[$NEXT_INDEX]}
  elif [ ${ARGUMENTS[$INDEX]} == "-was_admin_password" ];then
    was_admin_password=${ARGUMENTS[$NEXT_INDEX]}
  elif [ ${ARGUMENTS[$INDEX]} == "-was_runas_user" ];then
    was_runas_user=${ARGUMENTS[$NEXT_INDEX]}
  elif [ ${ARGUMENTS[$INDEX]} == "?" ];then
    tsGetMessage.sh RFID05266I
    exit
  fi
done

# include the RFIDIC environment settings
SCRIPT_DIR=`dirname $0`
if [ -n "$RFIDIC_HOME" -a -d $RFIDIC_HOME"/bin" ];then
  . ${RFIDIC_HOME}/bin/tsEnv.sh
  . ${RFIDIC_HOME}/bin/utils.sh
  . ${RFIDIC_HOME}/bin/prereqChecks.sh
else
  . ${SCRIPT_DIR}/tsEnv.sh
  . ${SCRIPT_DIR}/utils.sh
  . ${SCRIPT_DIR}/prereqChecks.sh
fi

logFileName=TSDeployLog_`date "+%Y%m%d%H%M%S"`
assignLogFile ${scriptDir}/../logs/$logFileName
absLogPathName="`cd \"$logDir\" 2>/dev/null && pwd || echo \"$logDir\"`/$logFileName"

echo "-------- Starting importAddTSAR ------------"   >> $utilsLogFile 2>&1

getWAScredentials()
{
    if [ -n "$was_admin_user" -a -z "$was_admin_password" ];then
        tsGetMessage.sh RFID05030I "$was_admin_user"
        stty -echo
        read was_admin_password
        stty echo
        echo
    fi

    if [ -n "$was_admin_password" ];then
        export was_admin_password
    fi

    if [ -n "$was_admin_user" -a -n "$was_admin_password" ];then
        credentials="-user $was_admin_user -password $was_admin_password"
    else
        credentials=""
    fi
}

# Starting applications
# Repeat message here so it gets written to the log.
startApplications()
{
    log " " 
    log "-------- Starting TS applications --------"
    getWAScredentials
    
    ## assign WAS security credentials
    if [ -n "$was_admin_user" -a -n "$was_admin_password" ];then
      parms="-was_admin_user=$was_admin_user -was_admin_password=$was_admin_password"
    else
      parms=""
    fi
        
    ${TS_HOME}/bin/startTS.sh -components=ENGINE,UI,SERIALID,CAPTURE $parms >> $utilsLogFile 2>&1
    startTS_retCode=$?
    log "Return code from start applications is " $startTS_retCode
}

# Stopping applications
# Repeat message here so it gets written to the log.
stopApplications()
{
    log " "
    log "-------- Stopping TS applications --------"
    getWAScredentials
    
    ## assign WAS security credentials
    if [ -n "$was_admin_user" -a -n "$was_admin_password" ];then
      parms="-was_admin_user=$was_admin_user -was_admin_password=$was_admin_password"
    else
      parms=""
    fi
        
    ${TS_HOME}/bin/stopTS.sh -components=ENGINE,UI,SERIALID,CAPTURE $parms >> $utilsLogFile 2>&1
    stopTS_retCode=$?
    log "Return code from stop applications is " $stopTS_retCode
}

# Check if WAS credentials are correct
checkWASCredentials()
{
    getWAScredentials
    serverStatus=`${WAS_HOME}/bin/serverStatus.sh $SERVERNAME $credentials`

    present=`echo "$serverStatus" | grep "ADMN0022E"`
    status=$?
}

# Check if WAS is already started
checkWASStart()
{
  echo 
  echo "-------- Check if WAS is running ---------"
  if [ -z "$was_admin_user" -a -z "$was_admin_password" ];then
    isWASRunning
  else
    isWASRunning $was_admin_user $was_admin_password
  fi 
  was_status=$?
  echo "Return code from WAS check is " $was_status
}

# Display WAS startup failure messages 
wasStartupFailureMessages()
{
  if [ "$was_status" -eq "$TRUE" ];then
    if [ $startTS_retCode -ne "0" ];then
      echo
      tsGetMessage.sh -noid RFID97080E
      echo
      tsGetMessage.sh -noid RFID97081E $absLogPathName
      echo
      exit $startTS_retCode
    fi
  fi
}

# Create masterdata object caches 
createObjectCachesForMDM()
{
  echo "-------- Creating object caches ----------"
  createObjectCaches
}


checkLogs
appendClassPathWithWASJars

#Change SSL settings in tsEnv.sh file under the sslWASSettings function
SSL_SETTINGS=`sslWASSettings`

ALL_RFIDIC_DEFINES="-DRFIDIC_HOME=${RFIDIC_HOME} -DTS_HOME=${RFIDIC_HOME} $RFIDIC_MEDIUM_JAVA_HEAP $SSL_SETTINGS"

checkWASCredentials
if [ $status = "0" ];then
  echo
  tsGetMessage.sh RFID50508W
  echo
  exit 1 
fi

# Verifying 
java -classpath $CLASSPATH ${ALL_RFIDIC_DEFINES} com.ibm.ts.deployment.DeployCommand -importAdd -verify $@
RET_CODE=$?
echo "Return code from verify is " $RET_CODE >> $utilsLogFile 2>&1

# Display usage if arguments are incorrect
if [ $RET_CODE -eq "14" ];then
    echo
    tsGetMessage.sh RFID05266I
    echo
    exit $RET_CODE
fi

# Display message if system is invalid 
if [ $RET_CODE -eq "12" ];then
  echo
  tsGetMessage.sh -noid RFID97077W
  promptUserYesNo
  if [ "$?" -eq "$NO" ];then
    echo
    tsGetMessage.sh -noid RFID97072I
    echo
    exit 1
  fi
fi

# Exit on any other error
if [ $RET_CODE -ne "0" ];then
    exit $RET_CODE
fi

# Stop WAS apps if WAS is running
checkWASStart >> $utilsLogFile 2>&1

if [ "$was_status" -eq "$TRUE" ];then
  echo
  echo "-------- Stopping TS applications --------"
  stopApplications
else
  echo
  tsGetMessage.sh -noid RFID97082I
  echo
fi

# Import the artifacts 
echo
echo "-------- Importing TS artifacts ----------"
echo                                                >> $utilsLogFile 2>&1
echo "-------- Importing TS artifacts ----------"   >> $utilsLogFile 2>&1
java -classpath $CLASSPATH ${ALL_RFIDIC_DEFINES} com.ibm.ts.deployment.DeployCommand -importAdd $@   >> $utilsLogFile 2>&1
RET_CODE=$?
echo "Return code from import add is " $RET_CODE >> $utilsLogFile 2>&1

# Create any required object caches based on new metadata if import
# succeeded or succeeded with warnings
if [ $RET_CODE -eq "0" -o $RET_CODE -eq "99" ];then
  createObjectCachesForMDM >> $utilsLogFile 2>&1
fi

# Start the WAS apps if WAS was originally running 
if [ "$was_status" -eq "$TRUE" ];then
  echo
  echo "-------- Starting TS applications --------"
  startApplications
fi

# Return code 0 means success 
if [ $RET_CODE -eq "0" ];then
  wasStartupFailureMessages
  echo
  tsGetMessage.sh -noid RFID97079I
  echo
else  
  # Return code 99 means success but warnings were added to log. 
  # Set return code to 0 so we exit with success.
  if [ $RET_CODE -eq "99" ];then
    wasStartupFailureMessages
    echo
    tsGetMessage.sh RFID97090W $absLogPathName
    echo
    RET_CODE=0;
  
  # Any other return code means an error occurred  
  else
    echo
    tsGetMessage.sh RFID97081E $absLogPathName
    echo
  fi
fi

exit $RET_CODE
