/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.delivery;

import java.util.Iterator;
import java.util.List;

import javax.mail.internet.InternetAddress;

import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.ts.alerts.utils.TextMessageBuilder;

/**
 * A concrete implementation of {@link IDeliveryMethod} that delivers an alert
 * via an e-mail message.  It uses
 * {@link TextMessageBuilder} to customize a message template provided as a parameter.
 * 
 * @see Email
 * 
 * @author Ralf Rantzau (rrantza@us.ibm.com)
 */
public class EmailDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String    NAME = "Email delivery method"; //$NON-NLS-1$
  private static final String    DESCRIPTION = "Writes notification into an email."; //$NON-NLS-1$

  /**
   * Parameter for recipient's e-mail address(es) ("tjwatson@us.ibm.com"). If
   * multiple addresses are provided then they must be a comma-separated list.
   * 
   * <p><b>Name:</b> To
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   * @see InternetAddress#parse()
   */
  public static final IParameter TO_PARAMETER;
  /**
   * Parameter for message subject
   * 
   * <p><b>Name:</b> Subject
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter SUBJECT_PARAMETER;
  /**
   * Parameter for sender ("rrantza@us.ibm.com")
   * 
   * <p><b>Name:</b> From
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter FROM_PARAMETER;
  /**
   * Optional parameter for sender's personal name ("Ralf Rantzau <rrantza@us.ibm.com>")
   * 
   * <p><b>Name:</b> From personal
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> false
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter FROM_PERSONAL_PARAMETER;
  /**
   * Optional parameter for secondary recipient e-mail addresses.
   * A comma-separated list.
   * 
   * <p><b>Name:</b> CC
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> false
   * <br><b>Default value:</b> <em>none</em>
   * @see InternetAddress#parse()
   */
  public static final IParameter CC_PARAMETER;
  /**
   * Optional parameter for blind-copy recipient e-mail addresses.
   * A comma-separated list.
   * 
   * <p><b>Name:</b> BCC
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> false
   * <br><b>Default value:</b> <em>none</em>
   * @see InternetAddress#parse()
   */
  public static final IParameter BCC_PARAMETER;
  /**
   * Optional parameter for message priority.  Value is a string
   * specifying the priority.  "1" being highest, "3" 'normal', and
   * "5" being lowest.
   * 
   * <p><b>Name:</b> Priority
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> false
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter PRIORITY_PARAMETER;
  /**
   * Optional parameter naming the e-mail folder in which to save the
   * outbound message.
   * 
   * <p><b>Name:</b> Record folder
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> false
   * <br><b>Default value:</b> <em>none</em>
   * @see javax.mail.Folder
   */
  public static final IParameter RECORD_FOLDER_PARAMETER;
  /**
   * Required parameter for message text.
   * 
   * <p><b>Name:</b> Message
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 32000
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter MESSAGE_PARAMETER;
  
  private static final IParameter[] parameters;
  
  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    IAttributeType stringType = typeFactory.getStringType(50, 0);
    IAttributeType longStringType = typeFactory.getStringType(com.ibm.rfidic.alerts.impl.Constants.MAXIMUM_SIZE_FOR_LONGSTRING_TYPE, 0);
    
    TO_PARAMETER = new Parameter("To", stringType, true, null); //$NON-NLS-1$
    SUBJECT_PARAMETER = new Parameter("Subject", stringType, true, null); //$NON-NLS-1$
    FROM_PARAMETER = new Parameter("From", stringType, true, null); //$NON-NLS-1$
    FROM_PERSONAL_PARAMETER = new Parameter("From personal", stringType, false, null); //$NON-NLS-1$
    CC_PARAMETER = new Parameter("CC", stringType, false, null); //$NON-NLS-1$
    BCC_PARAMETER = new Parameter("BCC", stringType, false, null); //$NON-NLS-1$
    PRIORITY_PARAMETER = new Parameter("Priority", stringType, false, null); //$NON-NLS-1$
    RECORD_FOLDER_PARAMETER = new Parameter("Record folder", stringType, false, null); //$NON-NLS-1$
    MESSAGE_PARAMETER = new Parameter("Message", longStringType, true, null); //$NON-NLS-1$
    
    parameters = new IParameter[] {
        TO_PARAMETER,
        SUBJECT_PARAMETER,
        FROM_PARAMETER,
        FROM_PERSONAL_PARAMETER,
        CC_PARAMETER,
        BCC_PARAMETER,
        PRIORITY_PARAMETER,
        RECORD_FOLDER_PARAMETER,
        MESSAGE_PARAMETER
      };
  }
  
  private TextMessageBuilder  messageBuilder;

  public EmailDeliveryMethod()
  {
    super(NAME, DESCRIPTION);

    messageBuilder = new TextMessageBuilder();      
  }

  public IParameter[] getParameters()
  {
    return parameters;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#processNotifications(java.util.List, com.ibm.rfidic.alerts.IDeliveryPoint)
   */
  public void processNotifications(List<INotification> notifications, IDeliveryPoint deliveryPoint)
  {
    if( notifications == null || notifications.size() == 0 ) return;

    // we group the notifications that are sent for one subscription into one email.
    StringBuffer emailMessage = new StringBuffer();
    ISubscription subscription = null;
    Email email = null;
    
    for( Iterator<INotification> ix = notifications.iterator(); ix.hasNext(); )
    {
      INotification notification = ix.next();

      ISubscription sub = notification.getSubscription();
      if( sub != subscription )
      {
        if( email != null )
        {
          email.setText( emailMessage.toString() );
          email.send();
        }
        email = null;
        subscription = sub;
        emailMessage = new StringBuffer();
      }
      
      try
      {
        final IParameterValue[] parameterValues = notification.getDeliveryValues(deliveryPoint);
        final String message = (String) Utility.findValue( parameterValues, MESSAGE_PARAMETER ).getSQLValue();
        String text = messageBuilder.buildMessage(notification, message);
        
        emailMessage.append(text);
        
        if( email == null )
        {
          
          final String to = (String) Utility.findValue( parameterValues, TO_PARAMETER ).getSQLValue();
          final String subject = (String) Utility.findValue( parameterValues, SUBJECT_PARAMETER ).getSQLValue();
          final String from = (String) Utility.findValue( parameterValues, FROM_PARAMETER ).getSQLValue();
          
          IAttributeValue value = Utility.findValue( parameterValues, FROM_PERSONAL_PARAMETER );
          final String fromPersonal = value == null ? null : (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, CC_PARAMETER );
          final String cc = value == null ? null: (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, BCC_PARAMETER );
          final String bcc = value == null ? null: (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, PRIORITY_PARAMETER );
          final String priority = value == null ? null: (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, RECORD_FOLDER_PARAMETER );
          final String recordFolder = value == null ? null: (String) value.getSQLValue();
        
          email = new Email(to, subject, from, fromPersonal, cc, bcc, priority, recordFolder, null);
        } 
      }
      catch(RFIDICException e)
      {
        e.printStackTrace();
      }
      
      if( email != null )
      {
        email.setText( emailMessage.toString() );
        email.send();
      }
    }
  }

  public String toString()
  {
    String text = ""; //$NON-NLS-1$
    text += "["; //$NON-NLS-1$
    text += "id: " + getId() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "className: " + this.getClass().getName() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "name: " + getName() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "description: " + getDescription() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "]"; //$NON-NLS-1$
    return text;
  }

  public String[] getOutputChannels()
  {
    return null;
  }
}
