/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.delivery;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.rfidic.value.ValueFactory;


/**
 * An instance of {@link IDeliveryMethod} that counts and sends notifications to a regular
 * output channel.  If the number of notifications exceeds a threshold value, a new notification
 * is sent to a second, escalation, channel.
 * 
 * <p>For examples of configuring <code>EscalationDeliveryMethod</code> see:
 * <ul>
 *   <li>samples/alerts/src/com/ibm/rfidic/alerts/delivery/EscalationEmailNotifConfig.xml</li>
 *   <li>samples/alerts/src/com/ibm/rfidic/alerts/delivery/EscalationScheduledEmailNotifConfig.xml</li>
 * </ul>
 */
public class EscalationDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String       REGULAR_CHANNEL = "regularDelivery"; //$NON-NLS-1$
  private static final String       ESCALATION_CHANNEL = "escalationDelivery"; //$NON-NLS-1$
  private static final String       NAME = "Escalation delivery method"; //$NON-NLS-1$
  private static final String       DESCRIPTION = "Send the notifications on the regular output channel and counts them at the same time." + //$NON-NLS-1$
                                                  " If the number of notifications received in one day exceeds the given threshold, a new" + //$NON-NLS-1$
                                                  " notification is sent on the escalation channel."; //$NON-NLS-1$

  /**
   * Parameter for escalation threshold.  An integer value -- if the
   * number of notifications in a single day exceeds the threshold value,
   * an escalation notification is sent as well.
   * 
   * <p><b>Name:</b> Threshold
   * <br><b>Type:</b> int
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    THRESHOLD_PARAMETER;
  /**
   * Parameter used in creating the escalation message. Value is the
   * number of notifications sent on the regular channel on the current day.
   * 
   * <p><b>Name:</b> Notification_count
   * <br><b>Type:</b> int
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    ESCALATION_PARAMETER;
  
  private static final IParameter[] parameters;
  
  private static final String[]     outputChannels = new String[] { REGULAR_CHANNEL, ESCALATION_CHANNEL };
  
  private Calendar                  currDate = Calendar.getInstance();
  private  int                      notificationsCount = 0;
  private boolean                   escalationSentToday = false;
  
  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    final IAttributeType intType = typeFactory.getIntegerType();

    THRESHOLD_PARAMETER = new Parameter("Threshold", intType, true, null); //$NON-NLS-1$
     
    parameters = new IParameter[] { THRESHOLD_PARAMETER };
    
    // this parameter is used to build the escalation notification data
    ESCALATION_PARAMETER = new Parameter("Notification_count", intType, true, null); //$NON-NLS-1$
  }
  
  public EscalationDeliveryMethod()
  {
    super(NAME, DESCRIPTION);
  }
  
  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#getOutputChannels()
   */
  public String[] getOutputChannels()
  {
    return outputChannels;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#getParameters()
   */
  public IParameter[] getParameters()
  {
    return parameters;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#processNotifications(java.util.List, com.ibm.rfidic.alerts.IDeliveryPoint)
   */
  public void processNotifications(List<INotification> notifications, IDeliveryPoint deliveryPoint)
  {
    if( notifications == null || notifications.size() == 0 ) return;
    
    // send the notifications on the regular channel
    IDeliveryPoint nextDP = deliveryPoint.getDeliveryPoint(REGULAR_CHANNEL);
    nextDP.processNotifications(notifications);
     
    // Get the threshold value - the following assumes that all received notifications
    // belong to the same subscription, which is true unless an earlier delivery point
    // groups the notifications from multiple subscriptions together
    INotification notification = (INotification) notifications.get(0);
    IParameterValue[] values = notification.getDeliveryValues(deliveryPoint);
    int threshold = -1;
    try
    {
      threshold = ( (Integer) Utility.findValue(values, THRESHOLD_PARAMETER).getSQLValue() ).intValue();
    }
    catch ( RFIDICException e )
    {
      e.printStackTrace();
      return;
    }
    
    // Count the notifications and see if the number of received notifications
    // for one day exceeds the threshold.
    // The following logic assumes that the notifications are received
    // in ascending chronological order.
    for( Iterator<INotification> ix = notifications.iterator(); ix.hasNext(); )
    {
      notification = ix.next();
      Calendar notifDate = Calendar.getInstance();
      notifDate.setTime( notification.getTimestamp() );
      
      
      
      if( notifDate.get(Calendar.DAY_OF_MONTH) == currDate.get(Calendar.DAY_OF_MONTH) &&
          notifDate.get(Calendar.MONTH) == currDate.get(Calendar.MONTH) &&
          notifDate.get(Calendar.YEAR) == currDate.get(Calendar.YEAR) )
      {
        if( ! escalationSentToday )
        {
          notificationsCount++;
        
          if( notificationsCount > threshold )
          {
            // build the escalation notification
            IParameterValue[] newValues = new IParameterValue[1];
            newValues[0] = new ParameterValue( ESCALATION_PARAMETER, ValueFactory.getValue(notificationsCount) );
            INotification newNotification = new Notification( notification.getSubscription(), newValues);
            List<INotification> newNotifications = new ArrayList<INotification>();
            newNotifications.add( newNotification );
          
            // send the newly created notification on the escalation output channel
            nextDP = deliveryPoint.getDeliveryPoint(ESCALATION_CHANNEL);
            nextDP.processNotifications( newNotifications );
            escalationSentToday = true;
          }
        }
      }
      else
      {
        currDate = notifDate;
        notificationsCount = 1;
        escalationSentToday = false;
      }
    }
  }
}
