/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.delivery;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;

import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.ts.alerts.utils.TextMessageBuilder;

/**
 * Notification delivery method that places notifications in a file.  It uses
 * {@link TextMessageBuilder} to customize a message template provided as a parameter.
 * 
 * @author vcrisan
 */

public class FileDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String    NAME = "File delivery method"; //$NON-NLS-1$
  private static final String    DESCRIPTION = "Writes notification into a file."; //$NON-NLS-1$
  
  /**
   * Required parameter identifying the file where notifications are to be stored
   * 
   * <p><b>Name:</b> Delivery file name
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 50
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    FILE_NAME_PARAMETER;
  /**
   * Required parameter for message template text
   * 
   * <p><b>Name:</b> Message
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 32000
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    MESSAGE_PARAMETER;

  private static final IParameter[] parameters;

  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    final IAttributeType stringType = typeFactory.getStringType(255, 0);
    final IAttributeType longStringType = typeFactory.getStringType(com.ibm.rfidic.alerts.impl.Constants.MAXIMUM_SIZE_FOR_LONGSTRING_TYPE, 0);

    FILE_NAME_PARAMETER = new Parameter("Delivery file name", stringType, true, null); //$NON-NLS-1$
    MESSAGE_PARAMETER = new Parameter("Message", longStringType, true, null); //$NON-NLS-1$

    parameters = new IParameter[] { FILE_NAME_PARAMETER, MESSAGE_PARAMETER };
  }
  
  private TextMessageBuilder messageBuilder;

  /**
   * Default, no-args constructor
   */
  public FileDeliveryMethod()
  {
    super(NAME, DESCRIPTION);
    messageBuilder = new TextMessageBuilder();
  }

  public IParameter[] getParameters()
  {
    return parameters;
  }

  public void processNotifications(List<INotification> notifications, IDeliveryPoint deliveryPoint)
  {
    if ( notifications == null || notifications.size() == 0 )
      return;

    for ( Iterator<INotification> ix = notifications.iterator(); ix.hasNext(); )
    {
      INotification notification = ix.next();

      try
      {
        final IParameterValue[] parameterValues = notification.getDeliveryValues(deliveryPoint);
        
        final String deliveryFileName = (String) Utility.findValue(parameterValues, FILE_NAME_PARAMETER).getSQLValue();
        final String message = (String) Utility.findValue(parameterValues, MESSAGE_PARAMETER).getSQLValue();

        String text = messageBuilder.buildMessage(notification, message);

        File file = new File(deliveryFileName);

        final PrintWriter writer = new PrintWriter(new FileWriter(file, true));
        writer.print(text);

        writer.flush();
        writer.close();
      }
      catch ( IOException e )
      {
        e.printStackTrace();
      }
      catch(RFIDICException e)
      {
        e.printStackTrace();
      }
    }
  }

  public String toString()
  {
    String text = ""; //$NON-NLS-1$
    text += "[\n"; //$NON-NLS-1$
    text += "id: " + getId() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "className: " + this.getClass().getName() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "name: " + getName() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "description: " + getDescription() + ",\n"; //$NON-NLS-1$ //$NON-NLS-2$
    text += "]"; //$NON-NLS-1$
    return text;
  }

  public String[] getOutputChannels()
  {
    return null;
  }
}
