/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.delivery;

import java.util.Iterator;
import java.util.List;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.ts.alerts.utils.TextMessageBuilder;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.TypeFactory;

/**
 * Delivery method that sends notifications on a JMS queue.   It uses
 * {@link TextMessageBuilder} to customize a message template provided as a parameter.
 */
public class JMSDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String    NAME = "JMS delivery method"; //$NON-NLS-1$
  private static final String    DESCRIPTION = "Writes notifications into a JMS queue."; //$NON-NLS-1$

  /**
   * Required parameter for JNDI name for the JMS queue connection factory.  This connection
   * factory must be configured in the J2EE container (WebSphere Application Server).
   * <p>Example: jms/alert_cf
   * 
   * <p><b>Name:</b> JNDI name of queue connection factory
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 255
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    JNDI_QCF_PARAMETER;
  /**
   * Required parameter for the JNDI name for the JMS queue  This queue
   * must be configured in the J2EE container (WebSphere Application Server).
   * <p>Example: jms/alert_q
   * 
   * <p><b>Name:</b> JNDI name of JMS queue
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 255
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    JNDI_Q_PARAMETER;
  /**
   * Required parameter for message template text
   * 
   * <p><b>Name:</b> Message
   * <br><b>Type:</b> string
   * <br><b>Max length:</b> 32000
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    MESSAGE_PARAMETER;

  private static final IParameter[] parameters;

  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    final IAttributeType stringType = typeFactory.getStringType(255, 0);
    final IAttributeType longStringType = typeFactory.getStringType(com.ibm.rfidic.alerts.impl.Constants.MAXIMUM_SIZE_FOR_LONGSTRING_TYPE, 0);

    JNDI_QCF_PARAMETER = new Parameter("JNDI name of queue connection factory", stringType, true, null); //$NON-NLS-1$
    JNDI_Q_PARAMETER = new Parameter("JNDI name of JMS queue", stringType, true, null); //$NON-NLS-1$
    MESSAGE_PARAMETER = new Parameter("Message", longStringType, true, null); //$NON-NLS-1$

    parameters = new IParameter[] { JNDI_QCF_PARAMETER, JNDI_Q_PARAMETER, MESSAGE_PARAMETER };
  }

  private TextMessageBuilder        messageBuilder;

  /**
   * Default, no-args constructor
   */
  public JMSDeliveryMethod()
  {
    super(NAME, DESCRIPTION);
    messageBuilder = new TextMessageBuilder();
  }

  public IParameter[] getParameters()
  {
    return parameters;
  }

  public void processNotifications(List<INotification> notifications, IDeliveryPoint deliveryPoint)
  {
    if ( notifications == null || notifications.size() == 0 )
      return;

    for ( Iterator<INotification> ix = notifications.iterator(); ix.hasNext(); )
    {
      INotification notification = (INotification) ix.next();

      try
      {
        final IParameterValue[] parameterValues = notification.getDeliveryValues(deliveryPoint);

        final String connectionFactoryName = (String) Utility.findValue(parameterValues, JNDI_QCF_PARAMETER).getSQLValue();
        final String queueName = (String) Utility.findValue(parameterValues, JNDI_Q_PARAMETER).getSQLValue();
        final String message = (String) Utility.findValue(parameterValues, MESSAGE_PARAMETER).getSQLValue();

        final InitialContext initialContext = new InitialContext();
        final ConnectionFactory connectionFactory = (ConnectionFactory) initialContext.lookup(connectionFactoryName);
        final Queue queue = (Queue) initialContext.lookup(queueName);

        if ( connectionFactory == null || queue == null )
        {
          /*
           * FIXME: Replace this by an error code.
           */
          System.err.println("The queue connection factory name, " + connectionFactoryName +  //$NON-NLS-1$
              ", or the queue name, " + queueName + ", are not properly defined."); //$NON-NLS-1$ //$NON-NLS-2$
        }
        else
        {
          String text = messageBuilder.buildMessage(notification, message);

          final Connection connection = connectionFactory.createConnection();
          connection.start();
          final Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
          final TextMessage outMessage = session.createTextMessage();
          outMessage.setText(text);

          /*
           * Put the message into the queue.
           */
          session.createProducer(queue).send(outMessage);

          session.close();
          connection.close();
        }
      }
      catch ( NamingException e )
      {
        e.printStackTrace();
      }
      catch ( JMSException e )
      {
        e.printStackTrace();
      }
      catch(RFIDICException e)
      {
        e.printStackTrace();
      }
      catch ( RuntimeException e )
      {
        e.printStackTrace();
      }
    }
  }

  public String[] getOutputChannels()
  {
    return null;
  }
}
