/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.delivery;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import com.ibm.rfidic.alerts.AlertsAdapter;
import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;


/**
 * Delivery method that queues up pending notifications and delivers them to the subscriber on
 * a schedule.  This method must be configured with an output channel, named <code>scheduledDelivery</code>, which will include a
 * delivery method that will perform the actual delivery.
 * 
 * <p>For examples of configuring <code>ScheduleDeliveryMethod</code> see:
 * <ul>
 *   <li>samples/alerts/src/com/ibm/rfidic/alerts/delivery/ScheduledEmailNotifConfig.xml</li>
 * </ul>
 */
public class ScheduleDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String       OUTPUT_CHANNEL = "scheduledDelivery"; //$NON-NLS-1$
  private static final String       NAME = "Schedule delivery method"; //$NON-NLS-1$
  private static final String       DESCRIPTION = "Schedules the notifications and sends them to the subscriber in bulk at the specified time."; //$NON-NLS-1$

  /**
   * Parameter for the hour of the day for notifications to be delivered
   * 
   * <p><b>Name:</b> Scheduled hour
   * <br><b>Type:</b> int
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    HOUR_PARAMETER;
  /**
   * Parameter for the minute of the hour for notifications to be delivered
   * 
   * <p><b>Name:</b> Scheduled minute
   * <br><b>Type:</b> int
   * <br><b>Required:</b> true
   * <br><b>Default value:</b> <em>none</em>
   */
  public static final IParameter    MINUTE_PARAMETER;
  private static final IParameter[] parameters;
  
  private static final String[]     outputChannels = new String[] { OUTPUT_CHANNEL };
  
  private static ConcurrentMap<IDeliveryPoint,List<INotification>> notificationMap = new ConcurrentHashMap<IDeliveryPoint,List<INotification>>();
  private static ScheduleThread scheduleThread = null;
  
  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    final IAttributeType intType = typeFactory.getIntegerType();

    HOUR_PARAMETER = new Parameter("Scheduled hour", intType, true, null); //$NON-NLS-1$
    MINUTE_PARAMETER = new Parameter("Scheduled minute", intType, true, null); //$NON-NLS-1$

    parameters = new IParameter[] { HOUR_PARAMETER, MINUTE_PARAMETER };
  }
  
  static class ScheduleThread extends Thread
  {
    public void run()
    {
      // loop forever, later inside the loop we might decide to bail out
      while( true )
      {
        try
        {
          Calendar now = Calendar.getInstance();
          int currHour = now.get(Calendar.HOUR_OF_DAY);
          int currMinute = now.get(Calendar.MINUTE);
          int currTime = currHour * 60 + currMinute;
          
            if( notificationMap.size() > 0 )
            {
              for( Iterator<IDeliveryPoint> ix = notificationMap.keySet().iterator(); ix.hasNext(); )
              {
                IDeliveryPoint deliveryPoint = ix.next();
                List<INotification> notifications = notificationMap.get(deliveryPoint);
                if( notifications.size() == 0 )
                {
                  ix.remove();
                }
                else
                {
                  // found some notifications pending, let's see if their scheduled time is now
                  // all notifications in the list belong to the same delivery point instance
                  // so that means that all of them have the same scheduled time
                  INotification notification = (INotification) notifications.get(0);
                  IParameterValue[] values = notification.getDeliveryValues(deliveryPoint);

                  IAttributeValue hourValue = Utility.findValue(values, HOUR_PARAMETER);
                  IAttributeValue minuteValue = Utility.findValue(values, MINUTE_PARAMETER);

                  int scheduledHour = ( (Integer) hourValue.getSQLValue() ).intValue();
                  int scheduledMinute = ( (Integer) minuteValue.getSQLValue() ).intValue();

                  Calendar scheduleTime = Calendar.getInstance();
                  scheduleTime.set(Calendar.HOUR_OF_DAY, scheduledHour);
                  scheduleTime.set(Calendar.MINUTE, scheduledMinute);
                  
                  // send the notifications further if we have reached or passed the scheduled time
                  // and only if the notifications have been received before the scheduled time 
                  if( currTime >= scheduledHour * 60 + scheduledMinute  &&
                      notification.getTimestamp().getTime() <= scheduleTime.getTimeInMillis() )
                  {
                    // first, remove these notifications from the map
                    ix.remove();

                    // find the delivery point that is mapped to the output channel
                    // and send the notifications to it
                    IDeliveryPoint nextDP = deliveryPoint.getDeliveryPoint(OUTPUT_CHANNEL);

                    if( nextDP != null )
                    {
                      nextDP.processNotifications(notifications);
                    }
                    else
                    {
                      throw new RFIDICException(this.getName() + ": Can't find the link for the outbound channel " + OUTPUT_CHANNEL); //$NON-NLS-1$
                    }
                  }
                }
            }
          }

          // sleep until next minute, but break it in 1 second chunks to be able to terminate if requested
          boolean terminate = false;
          int sec = 60 - Calendar.getInstance().get(Calendar.SECOND);
          for( int s = 0; s < sec; s++ )
          {
            // stop if there are no more pending notifications or if the alerts engine has stopped
            if( notificationMap.size() == 0 || AlertsAdapter.getInstance().isStarted() == false )
            {
              terminate = true;
              break;
            }
          
            Thread.sleep( 1000 );  // sleep one second
          }
          
          if( terminate ) break;
        }
        catch ( InterruptedException e )
        {
        }
        catch ( RFIDICException e )
        {
          e.printStackTrace();
        }
      }
    }
  }
  
  public ScheduleDeliveryMethod()
  {
    super(NAME, DESCRIPTION);
  }
  
  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#getOutputChannels()
   */
  public String[] getOutputChannels()
  {
    return outputChannels;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#getParameters()
   */
  public IParameter[] getParameters()
  {
    return parameters;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#processNotifications(java.util.List, com.ibm.rfidic.alerts.IDeliveryPoint)
   */
  public void processNotifications(List<INotification> notifications, IDeliveryPoint deliveryPoint)
  {
    if( notifications == null ) return;
    
      List<INotification> notifs = notificationMap.get(deliveryPoint);
      if( notifs == null )
      {
        notifs = new ArrayList<INotification>();
        notificationMap.put(deliveryPoint, notifs);
      }
      notifs.addAll(notifications);
      
      // start the scheduling thread was not started yet or it has terminated
      if( scheduleThread == null || ! scheduleThread.isAlive() )
      {
        scheduleThread = new ScheduleThread();
        scheduleThread.start();
      }
  }
}
