/*
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */

package com.ibm.ts.alerts.nativealerts;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.common.epcis.EventConstants;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.StringAttributeValue;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.rfidic.value.ValueFactory;
import com.ibm.ts.alerts.utils.DBUtil;

/**
 * This class is a sample Native Alert provided to show how a Native Alert
 * should be built. It sends a notification when an epc is commissioned a second
 * time. There are no input parameters but the epc, event time and location are
 * written as Alert output parameters to the Notification object.
 */
public class DuplicateCommissionAlert extends SuperNativeAlert 
{
  /* Declare arrays for the input and output parameters. */
  private IParameter[] inputParameters;

  private IParameter[] outputParameters;

  public DuplicateCommissionAlert() 
  {
    /*
     * Call the super class constructor passing in the name and description
     * of the alert. This text will appear in the Alerts Subscription wizard
     * pages.
     */
    super("Duplicate Commission alert", //$NON-NLS-1$
          "Detects a duplicate commission event for an epc. Exports $epc, $bizLocation and $eventTime. Please cut and paste these values for use in your alert message"); //$NON-NLS-1$

    /*
     * Create a type for the parameters. They will all be strings. The first
     * parameter (255) is maximum size and the second is reserved size.
     */
    IAttributeType stringType = 
      TypeFactory.getInstance().getStringType(255, 0);

    /*
     * Declare an input parameter array for our alert definition. These
     * parameters will be provided by the subscription and entered by the
     * user into the third page of the Alerts Subscription wizard.
     */
    this.inputParameters = new IParameter[1];
    
    this.inputParameters[0] = new Parameter (
        "bizStep",       // Name
        stringType,      // Type - string
        false,           // Required - no
        ValueFactory.getValue("urn:epcglobal:cbv:bizstep:commissioning"));
    
    /*
     * Declare an output parameter array for the Alert. These parameters
     * will be placed in the returned Notification and optionally used by
     * the Delivery Method.
     */
    this.outputParameters = new IParameter[3];

    this.outputParameters[0] = new Parameter(
        "epc",         // Name //$NON-NLS-1$
        stringType,    // Type - string
        true,          // Required - yes
        null);         // Default value - none

    this.outputParameters[1] = new Parameter(
        "bizLocation", // Name //$NON-NLS-1$
        stringType,    // Type - string
        true,          // Required - yes
        null);         // Default value - none

    this.outputParameters[2] = new Parameter(
        "eventTime",   // Name //$NON-NLS-1$
        stringType,    // Type - string
        true,          // Required - yes
        null);         // Default value - none
}

  /*
   * This method contains the condition to be evaluated for this Alert. It
   * returns a Notification.
   */
  protected List<INotification> evaluateCondition(IEvent event, ISubscription subscription)
  {
    System.out.println("event = "+event); //$NON-NLS-1$
    
    /* Retrieve the standard EPCIS eventTime attribute from the Event. */
    IAttributeValue bizStepValue = getAttributeValue(event, EventConstants.bizStep);
    String bizStep = (String) bizStepValue.getSQLValue();
    System.out.println("bizStep = "+bizStep); //$NON-NLS-1$
    
    final ArrayList<INotification> notifications = new ArrayList<INotification>();
    
    final String commBizStep = getParameterValue(subscription, "bizStep").toString ();
    
    /* if this bizStep is a commission bizStep */
    if (bizStep.equals(commBizStep))  //$NON-NLS-1$
    {
      IAttributeValue eventTimeValue   = null;
      IAttributeValue bizLocationValue = null;  
      
      /* Retrieve the standard EPCIS eventTime attribute from the Event. */
      eventTimeValue = getAttributeValue(event, EventConstants.eventTime);
      System.out.println("eventTimeValue = " + eventTimeValue); //$NON-NLS-1$

      java.sql.Timestamp eventTime = (java.sql.Timestamp) eventTimeValue.getSQLValue();
      System.out.println("eventTime = " + eventTime); //$NON-NLS-1$

      /* Retrieve the standard EPCIS bizLocation attribute from the Event. */
      bizLocationValue = getAttributeValue(event, EventConstants.bizLocation);
      System.out.println("bizLocationValue = " + bizLocationValue); //$NON-NLS-1$
      
      String bizLocation = (String) bizLocationValue.getSQLValue();
      System.out.println("bizLocation = " + bizLocation); //$NON-NLS-1$

      String epc               = null;
      IAttributeValue epcValue = null;
      String dsJndiName        = "jdbc/ts/tsDS"; //$NON-NLS-1$
      ResultSet result         = null;
      
      /* SELECT from db event for this epc and bizStep with a different
       * timestamp - could also use a slower wed service call.  */
      String query = "SELECT count(ve.event_id) count " + //$NON-NLS-1$
                     "FROM   viewschema.event ve, " + //$NON-NLS-1$
                     "       viewschema.epclist vel " + //$NON-NLS-1$
                     "where  ve.bizStep = '"+commBizStep+"' " + //$NON-NLS-1$
                     "and    ve.event_id = vel.event_id " + //$NON-NLS-1$
                     "and    vel.epc = ?"; //$NON-NLS-1$
      try
      {
        DBUtil dbUtil = new DBUtil();
        PreparedStatement pstmt = dbUtil.getPreparedStatement(query, dsJndiName);
      
        int epcNumber = 0;
        int count = 0;
        IParameterValue[] paramValues = null;
        do
        {
          // loop through epc's
          epcValue = getAttributeValue(event, EventConstants.epc, "#"+epcNumber); //$NON-NLS-1$
          System.out.println("epcValue = "+epcValue); //$NON-NLS-1$
          
          if(epcValue != null)
          {
            epc = (String) epcValue.getSQLValue();
            System.out.println("epc = "+epc); //$NON-NLS-1$
            
            pstmt.setString(1, epc);
            result = pstmt.executeQuery();
            
            System.err.println("Result Set: " + result); //$NON-NLS-1$

            if (result != null)
            {
              if(result.next())
              {
                count = result.getInt(1);
                
                System.out.println("count = " + count); //$NON-NLS-1$

                // if result then raise an alert for this epc and timestamp and
                // bizStep and old timestamp */
                if(count>1)
                {     
                  /*
                   * Create a parameter values array for use in the output
                   * notification.
                   */
                  paramValues = new ParameterValue[3];

                  /* Populate the array with parameters and their values */
                  paramValues[0] = new ParameterValue(outputParameters[0], epcValue);
                  paramValues[1] = new ParameterValue(outputParameters[1], bizLocationValue);
                  paramValues[2] = new ParameterValue(outputParameters[2], eventTimeValue);

                  final INotification note = new Notification (subscription, paramValues);
                  notifications.add(note);
                  System.out.println("added to notifications "+note); //$NON-NLS-1$
                }
              }
              result.close();
            }
            epcNumber++;
          }
        }
        while(epcValue != null);
        dbUtil.closeConnection();
      }
      catch(Exception e )
      {
        System.out.println("e = " + e);        //$NON-NLS-1$
        e.printStackTrace();
      }
    }
    
    System.out.println("notifications = "+notifications.size ()); //$NON-NLS-1$
    
    if(notifications.isEmpty())
    {
      /* Just return null if the condition is not true. */
      return null;
    } else {     
      /* Create and return the notifications for this subscription. */
      return notifications;
    }
  }

  /*
   * This method just returns the input parameters. There is no need to alter
   * this method for Native Alerts.
   */
  public IParameter[] getInputParameters() 
  {
    return inputParameters;
  }

  /*
   * This method just returns the output parameters. There is no need to alter
   * this method for Native Alerts.
   */
  public IParameter[] getOutputParameters() 
  {
    return outputParameters;
  }
}