/*
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.nativealerts;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import com.ibm.rfidic.alerts.AlertsAdapter;
import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.innate.INativeAlert;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.metadata.IAttributeMetaData;
import com.ibm.rfidic.metadata.IEntityMetaData;
import com.ibm.rfidic.path.IAttributePath;
import com.ibm.rfidic.path.PathDictionary;
import com.ibm.rfidic.value.IAttributeValue;

/** 
 * This class is an abstract class for all Native Alerts. It contains common
 * methods that are needed by all Native Alerts and do not need to be altered by
 * the Solution Developer.
 */
public abstract class SuperNativeAlert extends Entity implements INativeAlert
{
  /** Constructor */
  public SuperNativeAlert(String name, String description)
  {
    super(name, description);
  }

  /** This method is called after the constructor. 
   * There is no need to alter this method for Native Alerts. */
  public boolean initialize()
  {
    return true;
  }
  
  /**
   * This method is called when a new event arrives in the RFIDIC system. The
   * method is responsible for handling all subscriptions that have been made
   * to this alert. It returns a list of notifications to be passed to the
   * Delivery Method(s). This method may be altered for performance tuning 
   * if necessary. This method calls the abstract method evaluateCondition
   * which is overridden by a solution developer implementing a specific 
   * native alert.
   */
  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.innate.INativeAlert#evaluateConditions(com.ibm.rfidic.event.IEvent)
   */
  public List<INotification> evaluateConditions(IEvent event) 
  {
    final Collection<ISubscription> subscriptions = AlertsAdapter.getInstance()
        .getSubscriptionRegistry().getAllForAlert(getId());
    final List<INotification> notifications = new ArrayList<INotification>();
    for (Iterator<ISubscription> iterator = subscriptions.iterator(); iterator.hasNext();) 
    {
      final ISubscription subscription = (ISubscription) iterator.next();
      if(subscription.isEnabled())
      {
        List<INotification> returnNotifications = evaluateCondition(event, subscription);
        if (returnNotifications != null)
          notifications.addAll(returnNotifications);
      }
    }
    return notifications;
  }
  
  /** This abstract method must be provided by the solution developer
   * who implements a native alert in the alert class that extends this 
   * class. It returns a List of INotification objects which result from 
   * the processing of the alert based on the IEvent and ISubscription 
   * passed in or null may be returned if the alert condition is false.
   */
  abstract protected List<INotification> evaluateCondition(IEvent event, ISubscription subscription);

  /** This method handles subscriptions. 
   * There is no need to alter this method for Native Alerts. */
  public void addSubscription(ISubscription subscription)
  {
  }

  /** This method handles subscriptions. 
   * There is no need to alter this method for Native Alerts. */
  public void removeSubscription(ISubscription subscription)
  {
  }
  
  /** This method registers this Alert with the Native engine. 
   * There is no need to alter this method for Native Alerts. */
  public void register()
  {
    AlertsAdapter.getInstance().getNativeEngine().registerAlert(this);
  }

  /** This method unregisters this Alert with the Native engine. 
   * There is no need to alter this method for Native Alerts. */
  public void unregister()
  {
    AlertsAdapter.getInstance().getNativeEngine().unregisterAlert(this);
  }
  
  /** 
   * This is a standard method that will return an attribute value from an
   * event given the attribute's name. If the attribute being retrieved is a 
   * standard EPCIS attribute then the name should a value from the 
   * EventConstants object. Otherwise it should be the name of the 
   * extended attribute defined in the event metadata for the solution.  
   */
  public IAttributeValue getAttributeValue(IEvent event, String attrName) 
  {
    return(getAttributeValue(event, attrName, "")); //$NON-NLS-1$
  }

  /**
   * This method is an extension to the overloaded method getAttributeValue.  This method
   * allows the passing of a third parameter which is a String suffix to the attribute name.
   * This is used to retrieve attribute values for epcLists.  Since lists are suffixed
   * with #1, #2, etc, this String can be passed in to retrieve the individual elements in the list.
   */
  public IAttributeValue getAttributeValue(IEvent event, String attrName, String PathSuffix) 
  {
    IAttributeValue rval = null;
    final IEntityMetaData eventMD = event.getEntityMetaData();
    if (eventMD != null) {
      final IAttributeMetaData ia = eventMD.getAttributeMetaDataByName (attrName);
      if (ia != null) {
        String attrPath = eventMD.getAttributeAbsolutePath (ia) + PathSuffix;
        IAttributePath attributePath = PathDictionary.getInstance ().getAttributePath (attrPath);
        rval = event.getAttributeValue (attributePath);
      }
    }
    return rval;
  }
  
  /**
   * Answer an alert subscription parameter value or its default value.  Extracts a
   * parameter value from the subscription by that value's ordinal position.  If the
   * parameter has not been specified in the subscription, return the input paramter's
   * default value (if any.)
   * 
   * @param subscription Subscription containing alert parameters
   * @param paramIndex Index of parameter in list of parameters
   * @return Parameter value
   */
  protected IAttributeValue getParameterValue (ISubscription subscription, int paramIndex) {
    IAttributeValue rval = null;
    final IParameterValue[] pValues = subscription.getAlertParameterValues ();
    if (pValues != null) {
      if (paramIndex < 0 || paramIndex >= pValues.length) {
        throw new IllegalArgumentException ("Invalid paramIndex "+paramIndex);
      }
      if (pValues[paramIndex] != null) {
        rval = pValues[paramIndex].getValue ();
      }
    }
    if (rval == null) {
      rval = getInputParameters ()[paramIndex].getDefaultValue ();
    }
    if (rval == null) {
      throw new IllegalArgumentException ("Missing parameter "+paramIndex);
    }
    return rval;
  }
  
  /**
   * Answer an alert subscription parameter value or its default value.  Extracts a
   * parameter value from the subscription by that value's name.  If the
   * parameter has not been specified in the subscription, return the input paramter's
   * default value (if any.)
   * @param subscription Subscription containing alert parameters
   * @param paramName Name of the alert parameter
   * @return Parameter value
   */
  protected IAttributeValue getParameterValue (ISubscription subscription, String paramName) {
    IAttributeValue rval = null;
    final IParameterValue pValue = subscription.getAlertParameterValue (paramName);
    if (pValue != null) {
      rval = pValue.getValue ();
    }
    if (rval == null) {
      for (IParameter param: getInputParameters ()) {
        if (paramName.equals (param.getName ())) {
          rval = param.getDefaultValue ();
        }
      }
    }
    if (rval == null) {
      throw new IllegalArgumentException ("Missing parameter "+paramName);
    }
    return rval;
  }
}
